@@ -42,8 +42,38 @@ $EXPORT_TAGS{ all } = [@EXPORT_OK];
 
 our $DIE_ON_ERROR = 0;
 our $PROMOTE_N32 = 1;
+our $REPAIR_V3_FORMAT = 0;
+our $WARN_ON_REPAIR = 1;
 
-our $VERSION = '4.001';
+our $VERSION = '4.002';
+
+our $fourish = qr/^(?:::ffff:0:)?(\d+)\.(\d+)\.(\d+)\.(\d+)$/io;
+our $broken_fourish = qr/^::ffff:(\d+)\.(\d+)\.(\d+)\.(\d+)$/io;
+our $numberish = qr/^\d+$/o;
+our $normalish = qr/^([0-9a-f]{32})$/io;
+our $sixish = qr/^([0-9a-f:]+)(?:\%.*)?$/io;
+
+sub _repair_v3_format {
+  my ($old) = @_;
+  if (
+    $old->[ 8 ] == 0
+    && $old->[ 9 ] == 0
+    && $old->[ 10 ] == 0xff
+    && $old->[ 11 ] == 0xff
+  ) {
+    if ($WARN_ON_REPAIR > 1) {
+      cluck('Repairing v3.x module data to v4.x data');
+    }
+    elsif ($WARN_ON_REPAIR) {
+      carp('Repairing v3.x module data to v4.x data');
+    }
+    $old->[ 8 ] = 0xff;
+    $old->[ 9 ] = 0xff;
+    $old->[ 10 ] = 0;
+    $old->[ 11 ] = 0;
+  }
+  return $old;
+}
 
 sub IP {
   return Net::IPAddress::Util->new($_[0]);
@@ -62,13 +92,12 @@ sub new {
   }
   elsif (ref($address) eq 'ARRAY' && @$address == 4) {
     # FIXME Principal of least surprise here? Should feeding in 4 values make an IPv4?
-    # TODO At least document what it's doing, to reduce the surprise.
     $normal = [ unpack 'C16', pack 'N4', @$address ];
   }
   elsif (ref $address and eval { $address->isa(__PACKAGE__) }) {
-    return bless { address => $address->{ address } } => $class;
+    $normal = [ unpack 'C16', $address->{ address } ];
   }
-  elsif ($address =~ /^(?:::ffff:0:)?(\d+)\.(\d+)\.(\d+)\.(\d+)$/o) {
+  elsif ($address =~ $fourish) {
     $normal = [
       0, 0, 0, 0,
       0, 0, 0, 0,
@@ -76,7 +105,21 @@ sub new {
       $1, $2, $3, $4
     ];
   }
-  elsif ($PROMOTE_N32 and $address =~ /^\d+$/o and $address >= 0 and $address <= (2 ** 32) - 1) {
+  elsif ($REPAIR_V3_FORMAT && $address =~ $broken_fourish) {
+    if ($WARN_ON_REPAIR > 1) {
+      cluck('Repairing v3.x module data to v4.x data');
+    }
+    elsif ($WARN_ON_REPAIR) {
+      carp('Repairing v3.x module data to v4.x data');
+    }
+    $normal = [
+      0, 0, 0, 0,
+      0, 0, 0, 0,
+      0xff, 0xff, 0, 0,
+      $1, $2, $3, $4
+    ];
+  }
+  elsif ($PROMOTE_N32 and $address =~ $numberish and $address >= 0 and $address <= (2 ** 32) - 1) {
     $normal = [
       0, 0, 0, 0,
       0, 0, 0, 0,
@@ -84,7 +127,7 @@ sub new {
       unpack('C4', pack('N', $address))
     ];
   }
-  elsif ("$address" =~ /^([0-9a-f]{32})$/smio) {
+  elsif ("$address" =~ $normalish) {
     my $fresh = $1;
     eval "require Math::BigInt" or return ERROR("Could not load Math::BigInt: $@");
     my $raw = Math::BigInt->from_hex("$fresh");
@@ -98,7 +141,7 @@ sub new {
     }
     eval "no Math::BigInt";
   }
-  elsif ($address =~ /^\d+$/o) {
+  elsif ($address =~ $numberish) {
     eval "require Math::BigInt" or return ERROR("Could not load Math::BigInt: $@");
     my $raw = Math::BigInt->new("$address");
     while ($raw > 0) {
@@ -112,11 +155,11 @@ sub new {
     eval "no Math::BigInt";
   }
   elsif (
-    (
+    $address =~ $sixish
+    and (
       scalar(grep { /::/o } split(/[[:alnum:]]+/, $address)) == 1
       or scalar(grep { /[[:alnum:]]+/ } split(/:/, $address)) == 8
     )
-    and $address =~ /^([0-9a-f:]+)(?:\%.*)?$/msoi
   ) {
     # new() from IPv6 address, accepting and ignoring the Scope ID
     $address = $1;
@@ -133,11 +176,14 @@ sub new {
     }
   }
   elsif (length($address) == 16) {
-    return bless { address => $address } => $class;
+    $normal = [ unpack('C16', $address) ];
   }
   else {
     return ERROR("Invalid argument `$address', a(n) " . (ref($address) || 'bare scalar') . ' provided');
   }
+  if ($REPAIR_V3_FORMAT) {
+    $normal = _repair_v3_format($normal);
+  }
   return bless { address => pack('C16', @$normal) } => $class;
 }
 
@@ -532,7 +578,7 @@ Net::IPAddress::Util - Version-agnostic representation of an IP address
 
 =head1 VERSION
 
-Version 4.001
+Version 4.002
 
 =head1 SYNOPSIS
 
@@ -560,8 +606,8 @@ The goal of the Net::IPAddress::Util modules is to make IP addresses easy to
 deal with, regardless of whether they're IPv4 or IPv6, and regardless of the
 source (and destination) of the data being manipulated. The module
 Net::IPAddress::Util is for working with individual addresses,
-Net::IPAddress::Util::Range is for working with individual ranges of
-addresses, and Net::IPAddress::Util::Collection is for working with
+L<Net::IPAddress::Util::Range> is for working with individual ranges of
+addresses, and L<Net::IPAddress::Util::Collection> is for working with
 collections of addresses and/or ranges.
 
 =head1 GLOBAL VARIABLES
@@ -579,42 +625,82 @@ accordingly (i.e. to do implicitly what n32_to_ipv4() does). Set to a false
 value to make new() treat all bare numbers as 128-bit numbers representing
 IPv6 addresses. Defaults to false.
 
+=head2 $Net::IPAddress::Util::REPAIR_V3_FORMAT
+
+Set to a true value to make new() accept its argument in the "broken" format
+used by module versions prior to v4.000, and automatically "repair" them to the
+format used by v4.x of this module. Defaults to false.
+
+=head2 $Net::IPAddress::Util::WARN_ON_REPAIR
+
+Set to a true value to make any "repairs" undertaken by new() issue a warning
+using the L<Carp> module. Specifically, if set to the number 2 (or higher), then
+new() will cluck() whenever it performs a repair, and if set to any other true
+value, then new() will carp() whenever it performs a repair. If set to a false
+value, then repairs (if any) will occur silently. Defaults to 1.
+
 =head1 EXPORTABLE FUNCTIONS
 
-=head2 explode_ip
+=over
 
-=head2 implode_ip
+=item explode_ip
+
+=item implode_ip
 
 Transform an IP address to and from an array of 128 bits, MSB-first.
 
-=head2 common_prefix
+=back
+
+=over
+
+=item common_prefix
 
 Given two bit arrays (as provided by C<explode_ip>), return the truncated
 bit array of the prefix bits those two arrays have in common.
 
-=head2 prefix_mask
+=back
+
+=over
+
+=item prefix_mask
 
 Given two bit arrays (as provided by C<explode_ip>), return a truncated bit
 array of ones of the same length as the shared C<common_prefix> of the two
 arrays.
 
-=head2 ip_pad_prefix
+=back
+
+=over
+
+=item ip_pad_prefix
 
 Take a truncated bit array, and right-pad it with zeroes to the appropriate
 length.
 
-=head2 ipv4_mask
+=back
+
+=over
+
+=item ipv4_mask
 
 Returns a bitmask that can be ANDed against an IP to pull out only
 the IPv4-relevant bits, that is the N32 portion with the 0xffff appended to its
 front.
 
-=head2 ipv4_flag
+=back
+
+=over
+
+=item ipv4_flag
 
 Returns a bitmask that can be ORed onto an N32 to make it a proper "IPv4
 stored as IPv6" N128.
 
-=head2 radix_sort
+=back
+
+=over
+
+=item radix_sort
 
 Given an array of objects, sorts them in ascending order, faster than Perl's
 built-in sort command.
@@ -625,17 +711,21 @@ duplicates, so ymmv. B<There are also (rare) corner cases> in which radix_sort()
 can chew up so much RAM that it causes paging / swapping, which I<will> slow
 down the process I<dramatically>.
 
+=back
+
 =head1 COMPATIBILITY API
 
-=head2 ip2num
+=over
 
-=head2 num2ip
+=item ip2num
 
-=head2 validaddr
+=item num2ip
 
-=head2 mask
+=item validaddr
 
-=head2 fqdn
+=item mask
+
+=item fqdn
 
 These functions are exportable to provide a functionally-identical API
 to that provided by L<Net::IPAddress>. They will cause warnings to be issued
@@ -644,6 +734,8 @@ if indeed that's what you're doing -- and I can't readily imagine any other
 reason you'd want to export them from here (as opposed to from Net::IPAddress)
 unless that's indeed what you're doing.
 
+=back
+
 =head1 EXPORT TAGS
 
 =head2 :constr
